<div class="input-group mb-3">
  <label class="sr-only" for="{{ form.password.id_for_label }}">{{ form.password.label }}</label>
  <input class="form-control"
         id="{{ form.password.id_for_label }}"
         name="{{ form.password.html_name }}"
         placeholder="{{ form.password.label }}"
         type="password"
         value="">
  <div class="input-group-append">
    <div class="input-group-text">
      <span class="fas fa-lock"></span>
    </div>
  </div>
  {% if form.password.errors %}
  <div class="invalid-feedback" id="{{ form.password.id_for_label }}-feedback">
    {{ form.password.errors.as_ul }}
  </div>
  {% endif %}
</div>

