{% extends 'dj_accounts/auth_base.html' %}
{% load i18n static %}

{% block content %}
<div class="card">
  <div class="card-body login-card-body">
    <p>{% translate 'Forgotten your password? Enter your email address below, and we’ll email instructions for setting a
      new one.' %}</p>

    <form method="post">
      {% csrf_token %}
      {% include "django/forms/form.html" with form=form %}
      <button class="btn btn-primary btn block" type="submit">{% translate 'Reset my password' %}</button>
    </form>
  </div>
</div>


{% endblock %}
