{% load i18n %}
<!DOCTYPE html>
<html lang="en">
<head>
  <meta charset="utf-8">
  <meta content="width=device-width, initial-scale=1" name="viewport">
  <title>{% translate "Accounts" %} | {% block page_title %}{% endblock %}</title>
  {% block styles %}{% endblock %}
</head>
<body>
{% block content %}{% endblock %}
</body>
</html>
