{% extends 'dj_accounts/auth_base.html' %}
{% load i18n static %}

{% block page_title %}
{% translate 'Password reset confirmation' %}
{% endblock %}

{% block content %}
<div class="card">
  <div class="card-body login-card-body">
    <p class="login-box-msg">
      {% translate "Password Reset" %}
    </p>
    {% if validlink %}
    <p class="login-box-msg">
      {% translate "Please enter your new password twice so we can verify you typed it in correctly." %}
    </p>
    <form method="post">
      {% csrf_token %}
      {% include "django/forms/form.html" with form=form %}
      <button class="btn btn-primary btn-block" type="submit">{% translate 'Change my password' %}</button>
    </form>
    {% else %}
    <p class="login-box-msg">
      {% translate "The password reset link was invalid, possibly because it has already been used. Please request a new
      password reset." %}
    </p>
    {% endif %}
  </div>
</div>
{% endblock %}
