<div class="input-group mb-3">
  <label class="sr-only" for="{{ form.phone.id_for_label }}">{{ form.phone.label }}</label>

  <input class="form-control"
         id="{{ form.phone.id_for_label }}"
         name="{{ form.phone.html_name }}"
         placeholder="{{ form.phone.label }}"
         type="text"
         value="{{ form.phone.value|default_if_none:'' }}">
  <div class="input-group-append">
    <div class="input-group-text">
      <span class="fas fa-phone"></span>
    </div>
  </div>
  {% if form.phone.errors %}
  <div class="invalid-feedback" id="{{ form.phone.id_for_label }}-feedback">
    {{ form.phone.errors.as_ul }}
  </div>
  {% endif %}
</div>