{% extends 'dj_accounts/profile_base.html' %}{% load static i18n %}
{% block content %}
<div class="container">
  <div class="row">
    <div class="col-lg-12">
      <div class="card">
        <div class="card__content">
          <div class="field-wrapper field-wrapper--lg">
            <form method="POST">
              {% csrf_token %}
              <div class="card-body">
                {% include 'django/forms/form.html' with form=form %}
                <button class="btn btn-primary" type="submit">{% translate 'Submit' %}</button>
                <a class="btn btn-default float-right" href="{% url 'site:profile:index' %}">{% translate 'Cancel'
                  %}</a>
              </div>
            </form>
          </div>
        </div>
      </div>
    </div>
  </div>
</div>

{% endblock %}