<div class="field-wrapper field-wrapper--full">
  <label class="field-wrapper__label" for="{{ form.code.id_for_label }}">{{ form.code.label }}</label>
  <div class="field-wrapper__content">
    <input class="field field--with-icon"
           id="{{ form.code.id_for_label }}"
           name="{{ form.code.html_name }}"
           placeholder="{{ form.code.label }}"
           type="text"
           value="{{ form.code.value|default_if_none:'' }}">
  </div>
  {% if form.code.errors %}
  <ul class="field-wrapper__messages">
    {% for error in field.errors %}
    <li>{{ error }}</li>
    {% endfor %}
  </ul>
  {% endif %}
</div>